@@ -351,6 +351,132 @@ impl<T: IdOrdItem> IdOrdMap<T> {
         self.tables.key_to_item.clear();
     }
 
+    /// Reserves capacity for at least `additional` more elements to be inserted
+    /// in the `IdOrdMap`. The collection may reserve more space to
+    /// speculatively avoid frequent reallocations. After calling `reserve`,
+    /// capacity will be greater than or equal to `self.len() + additional`.
+    /// Does nothing if capacity is already sufficient.
+    ///
+    /// Note: This only reserves capacity in the item storage. The internal
+    /// [`BTreeSet`] used for key-to-item mapping does not support capacity
+    /// reservation.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the new capacity overflows [`isize::MAX`] bytes, and
+    /// [`abort`]s the program in case of an allocation error.
+    ///
+    /// [`isize::MAX`]: https://doc.rust-lang.org/std/primitive.isize.html
+    /// [`abort`]: https://doc.rust-lang.org/alloc/alloc/fn.handle_alloc_error.html
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use iddqd::{IdOrdItem, IdOrdMap, id_upcast};
+    ///
+    /// #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
+    /// struct Item {
+    ///     id: String,
+    ///     value: u32,
+    /// }
+    ///
+    /// impl IdOrdItem for Item {
+    ///     type Key<'a> = &'a str;
+    ///     fn key(&self) -> Self::Key<'_> {
+    ///         &self.id
+    ///     }
+    ///     id_upcast!();
+    /// }
+    ///
+    /// let mut map: IdOrdMap<Item> = IdOrdMap::new();
+    /// map.reserve(100);
+    /// assert!(map.capacity() >= 100);
+    /// ```
+    pub fn reserve(&mut self, additional: usize) {
+        self.items.reserve(additional);
+    }
+
+    /// Shrinks the capacity of the map as much as possible. It will drop
+    /// down as much as possible while maintaining the internal rules
+    /// and possibly leaving some space in accordance with the resize policy.
+    ///
+    /// Note: This only shrinks the item storage capacity. The internal
+    /// [`BTreeSet`] used for key-to-item mapping does not support capacity
+    /// control.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use iddqd::{IdOrdItem, IdOrdMap, id_upcast};
+    ///
+    /// #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
+    /// struct Item {
+    ///     id: String,
+    ///     value: u32,
+    /// }
+    ///
+    /// impl IdOrdItem for Item {
+    ///     type Key<'a> = &'a str;
+    ///     fn key(&self) -> Self::Key<'_> {
+    ///         &self.id
+    ///     }
+    ///     id_upcast!();
+    /// }
+    ///
+    /// let mut map: IdOrdMap<Item> = IdOrdMap::with_capacity(100);
+    /// map.insert_unique(Item { id: "foo".to_string(), value: 1 }).unwrap();
+    /// map.insert_unique(Item { id: "bar".to_string(), value: 2 }).unwrap();
+    /// assert!(map.capacity() >= 100);
+    /// map.shrink_to_fit();
+    /// assert!(map.capacity() >= 2);
+    /// ```
+    pub fn shrink_to_fit(&mut self) {
+        self.items.shrink_to_fit();
+    }
+
+    /// Shrinks the capacity of the map with a lower limit. It will drop
+    /// down no lower than the supplied limit while maintaining the internal
+    /// rules and possibly leaving some space in accordance with the resize
+    /// policy.
+    ///
+    /// If the current capacity is less than the lower limit, this is a no-op.
+    ///
+    /// Note: This only shrinks the item storage capacity. The internal
+    /// [`BTreeSet`] used for key-to-item mapping does not support capacity
+    /// control.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use iddqd::{IdOrdItem, IdOrdMap, id_upcast};
+    ///
+    /// #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
+    /// struct Item {
+    ///     id: String,
+    ///     value: u32,
+    /// }
+    ///
+    /// impl IdOrdItem for Item {
+    ///     type Key<'a> = &'a str;
+    ///     fn key(&self) -> Self::Key<'_> {
+    ///         &self.id
+    ///     }
+    ///     id_upcast!();
+    /// }
+    ///
+    /// let mut map: IdOrdMap<Item> = IdOrdMap::with_capacity(100);
+    /// map.insert_unique(Item { id: "foo".to_string(), value: 1 }).unwrap();
+    /// map.insert_unique(Item { id: "bar".to_string(), value: 2 }).unwrap();
+    /// assert!(map.capacity() >= 100);
+    /// map.shrink_to(10);
+    /// assert!(map.capacity() >= 10);
+    /// map.shrink_to(0);
+    /// assert!(map.capacity() >= 2);
+    /// ```
+    pub fn shrink_to(&mut self, min_capacity: usize) {
+        self.items.shrink_to(min_capacity);
+    }
+
     /// Iterates over the items in the map.
     ///
     /// Similar to [`BTreeMap`], the iteration is ordered by [`T::Key`].
@@ -1203,17 +1329,23 @@ impl<T: IdOrdItem> IdOrdMap<T> {
                 (MapHash::new(hash), dormant_item)
             };
 
-            // SAFETY: The original items is no longer used after the first
-            // block above.
-            let items = unsafe { dormant_items.awaken() };
-            // SAFETY: The original item is no longer used after the second
-            // block above.
-            let item = unsafe { dormant_item.awaken() };
+            let retain = {
+                // SAFETY: The original item is no longer used after the second
+                // block above. dormant_items, from which item is derived, is
+                // currently dormant.
+                let item = unsafe { dormant_item.awaken() };
 
-            let ref_mut = RefMut::new(hash_state.clone(), hash, item);
-            if f(ref_mut) {
+                let ref_mut = RefMut::new(hash_state.clone(), hash, item);
+                f(ref_mut)
+            };
+
+            if retain {
                 true
             } else {
+                // SAFETY: The original items is no longer used after the first
+                // block above, and item + dormant_item have been dropped after
+                // being used above.
+                let items = unsafe { dormant_items.awaken() };
                 items.remove(index);
                 false
             }
@@ -1390,6 +1522,17 @@ impl<T: IdOrdItem + Eq> Eq for IdOrdMap<T> {}
 /// also be an `extend_unique` method that will return an error.
 impl<T: IdOrdItem> Extend<T> for IdOrdMap<T> {
     fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) {
+        // Keys may already be present in the map, or multiple times in the
+        // iterator. Reserve the entire hint lower bound if the map is empty.
+        // Otherwise reserve half the hint (rounded up), so the map will only
+        // resize twice in the worst case.
+        let iter = iter.into_iter();
+        let reserve = if self.is_empty() {
+            iter.size_hint().0
+        } else {
+            iter.size_hint().0.div_ceil(2)
+        };
+        self.reserve(reserve);
         for item in iter {
             self.insert_overwrite(item);
         }
