@@ -13,14 +13,14 @@
 //!
 //! ## Minimum supported Rust version
 //!
-//! The minimum supported Rust version (MSRV) is **Rust 1.78**.
+//! The minimum supported Rust version (MSRV) is **Rust 1.86**.
 //!
 //! While this crate is in pre-release status (0.x), the MSRV may be bumped in
 //! patch releases.
 
 #![warn(missing_docs)]
 #![forbid(unsafe_code)]
-#![cfg_attr(doc_cfg, feature(doc_cfg, doc_auto_cfg))]
+#![cfg_attr(doc_cfg, feature(doc_cfg))]
 
 #[cfg(feature = "fixtures")]
 pub mod fixtures;
